@@ -1,9 +1,11 @@
 import os
 from gettext import gettext as _
+from gettext import ngettext
 
 from gi.repository import Gio, Gtk
 
 from lutris import api, sysoptions
+from lutris.config import LutrisConfig
 from lutris.gui.config.add_game_dialog import AddGameDialog
 from lutris.gui.dialogs import ErrorDialog, ModelessDialog
 from lutris.gui.dialogs.game_import import ImportGameDialog
@@ -12,6 +14,8 @@ from lutris.gui.widgets.navigation_stack import NavigationStack
 from lutris.installer import AUTO_WIN32_EXE, get_installers
 from lutris.util.jobs import COMPLETED_IDLE_TASK, AsyncCall, schedule_at_idle
 from lutris.util.strings import gtk_safe, slugify
+from lutris.util.wine.proton import is_proton_version
+from lutris.util.wine.wine import GE_PROTON_LATEST
 
 
 class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
@@ -42,8 +46,8 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         (
             "application-x-firmware-symbolic",
             "go-next-symbolic",
-            _("Import a ROM"),
-            _("Import a ROM that is known to Lutris"),
+            _("Import ROMs"),
+            _("Import ROMs referenced in TOSEC, No-intro or Redump"),
             "import_rom",
         ),
         (
@@ -124,7 +128,9 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
 
         self.install_script_file_chooser = FileChooserEntry(title=_("Select script"), action=Gtk.FileChooserAction.OPEN)
 
-        self.import_rom_file_chooser = FileChooserEntry(title=_("Select ROM file"), action=Gtk.FileChooserAction.OPEN)
+        self.import_rom_file_chooser = FileChooserEntry(
+            title=_("Select ROMs"), action=Gtk.FileChooserAction.SELECT_FOLDER
+        )
 
         self.stack.add_named_factory("initial", self.create_initial_page)
         self.stack.add_named_factory("search_installers", self.create_search_installers_page)
@@ -253,7 +259,8 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         if not count:
             self.search_result_label.set_markup(_("No results"))
         elif count == total_count:
-            self.search_result_label.set_markup(_("Showing <b>%s</b> results") % count)
+            text = ngettext("Showing <b>%d</b> result", "Showing <b>%d</b> results", count) % count
+            self.search_result_label.set_markup(text)
         else:
             self.search_result_label.set_markup(_("<b>%s</b> results, only displaying first %s") % (total_count, count))
         for row in self.search_listbox.get_children():
@@ -312,12 +319,14 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         preset_label = Gtk.Label(_("Installer preset:"), visible=True)
         grid.attach(preset_label, 0, 3, 1, 1)
 
+        self.installer_presets.append(["win11", _("Windows 11 64-bit")])
         self.installer_presets.append(["win10", _("Windows 10 64-bit (Default)")])
         self.installer_presets.append(["win7", _("Windows 7 64-bit")])
-        self.installer_presets.append(["winxp", _("Windows XP 32-bit")])
-        self.installer_presets.append(["winxp-3dfx", _("Windows XP + 3DFX 32-bit")])
-        self.installer_presets.append(["win98", _("Windows 98 32-bit")])
-        self.installer_presets.append(["win98-3dfx", _("Windows 98 + 3DFX 32-bit")])
+
+        wine_version = LutrisConfig(runner_slug="wine").runner_config.get("version")
+        if wine_version != GE_PROTON_LATEST and not is_proton_version(wine_version):
+            self.installer_presets.append(["winxp", _("Windows XP 32-bit")])
+            self.installer_presets.append(["win98", _("Windows 98 32-bit")])
 
         renderer_text = Gtk.CellRendererText()
         self.install_preset_dropdown.pack_start(renderer_text, True)
@@ -404,8 +413,6 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         }
         if win_ver_task:
             installer["script"]["installer"].insert(0, win_ver_task)
-        if installer_preset.endswith("3dfx"):
-            installer["script"]["wine"] = {"dgvoodoo2": True}
         application = Gio.Application.get_default()
         application.show_installer_window([installer])
         self.destroy()
@@ -465,10 +472,10 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         self.import_rom_file_chooser.set_hexpand(True)
 
         explanation = _(
-            "Lutris will identify a ROM via its MD5 hash and download game "
+            "Lutris will identify ROMs via its MD5 hash and download game "
             "information from Lutris.net.\n\n"
-            "The ROM data used for this comes from the TOSEC project.\n\n"
-            "When you click 'Install' below, the process of installing the game will "
+            "The ROM data used for this comes from  TOSEC, No-Intro and Redump projects.\n\n"
+            "When you click 'Install' below, the process of installing the games will "
             "begin."
         )
 
@@ -476,19 +483,21 @@ class AddGamesWindow(ModelessDialog):  # pylint: disable=too-many-public-methods
         return grid
 
     def present_import_rom_page(self):
-        self.set_page_title_markup(_("<b>Select a ROM file</b>"))
+        self.set_page_title_markup(_("<b>Select ROMs</b>"))
         self.stack.present_page("import_rom")
         self.display_continue_button(self.on_continue_import_rom_clicked, label=_("_Install"))
 
     def on_continue_import_rom_clicked(self, _widget):
-        path = self.import_rom_file_chooser.get_text()
-        if not path:
-            ErrorDialog(_("You must select a ROM file to install."), parent=self)
-        elif not os.path.isfile(path):
-            ErrorDialog(_("No file exists at '%s'.") % path, parent=self)
+        paths = []
+        for path, _dirs, files in os.walk(self.import_rom_file_chooser.get_text()):
+            for file in files:
+                paths.append(os.path.join(path, file))
+
+        if not paths:
+            ErrorDialog(_("You must select ROMs to install."), parent=self)
         else:
             application = Gio.Application.get_default()
-            dialog = ImportGameDialog([path], parent=application.window)
+            dialog = ImportGameDialog(paths, parent=application.window)
             dialog.show()
             self.destroy()
 
