@@ -11,18 +11,12 @@ class AddGameDialog(GameDialogCommon):
         super().__init__(_("Add a new game"), config_level="game", parent=parent)
         self.game = game
         self.saved = False
-        if game:
-            self.runner_name = game.runner_name
-            self.slug = game.slug
-        else:
-            self.runner_name = runner
-            self.slug = None
 
         self.lutris_config = LutrisConfig(
-            runner_slug=self.runner_name,
+            runner_slug=game.runner_name if game else runner,
             level="game",
         )
         self.build_notebook()
         self.build_tabs()
-        self.name_entry.grab_focus()
+        self.info_box.grab_focus()
         self.show_all()
