@@ -10,8 +10,6 @@ class EditGameConfigDialog(GameDialogCommon):
         super().__init__(_("Configure %s") % game.name, config_level="game", parent=parent)
         self.game = game
         self.lutris_config = game.config
-        self.slug = game.slug
-        self.initial_slug = game.slug
         self.runner_name = game.runner_name
         self.build_notebook()
         self.build_tabs()
