@@ -4,6 +4,7 @@
 import os
 import traceback
 from gettext import gettext as _
+from typing import List
 
 from gi.repository import Gdk, Gio, GLib, Gtk
 
@@ -21,9 +22,11 @@ from lutris.gui.dialogs.cache import CacheConfigurationDialog
 from lutris.gui.dialogs.delegates import DialogInstallUIDelegate
 from lutris.gui.installer.files_box import InstallerFilesBox
 from lutris.gui.installer.script_picker import InstallerPicker
+from lutris.gui.widgets import NotificationSource
 from lutris.gui.widgets.common import FileChooserEntry
 from lutris.gui.widgets.log_text_view import LogTextView
 from lutris.gui.widgets.navigation_stack import NavigationStack
+from lutris.gui.widgets.utils import get_main_window
 from lutris.installer import InstallationKind, interpreter
 from lutris.installer.errors import MissingGameDependencyError, ScriptingError
 from lutris.installer.interpreter import ScriptInterpreter
@@ -32,19 +35,22 @@ from lutris.util.jobs import AsyncCall
 from lutris.util.linux import LINUX_SYSTEM
 from lutris.util.log import get_log_contents, logger
 from lutris.util.steam import shortcut as steam_shortcut
-from lutris.util.strings import human_size
+from lutris.util.strings import gtk_safe, human_size
 from lutris.util.system import is_removeable
 
+INSTALLATION_FAILED = NotificationSource()
+INSTALLATION_COMPLETED = NotificationSource()
+
 
 class MarkupLabel(Gtk.Label):
     """Label for installer window"""
 
     def __init__(self, markup=None, **kwargs):
-        super().__init__(label=markup, use_markup=True, wrap=True, max_width_chars=80, **kwargs)
+        super().__init__(label=markup, use_markup=True, wrap=True, justify=Gtk.Justification.CENTER, **kwargs)
         self.set_alignment(0.5, 0)
 
 
-class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter.InterpreterUIDelegate):  # pylint: disable=too-many-public-methods
+class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter.InterpreterUIDelegate):  # type:ignore[misc]
     """GUI for the install process.
 
     This window is divided into pages; as you go through the install each page
@@ -150,12 +156,12 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
         self.installer_files_box.connect("files-ready", self.on_files_ready)
 
         self.log_buffer = Gtk.TextBuffer()
+        self.error_details_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6, no_show_all=True)
         self.error_details_buffer = Gtk.TextBuffer()
         self.error_reporter = self.load_error_page
 
-        self.load_choose_installer_page()
-
         # And... go!
+        self.load_first_page()
         self.show_all()
         self.present()
 
@@ -217,6 +223,18 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
 
     def on_cancel_clicked(self, _button=None):
         """Ask a confirmation before cancelling the installation, if it has started."""
+
+        def on_cancelled():
+            if self.interpreter:
+                self.interpreter.cleanup()  # still remove temporary downloads in any case
+
+            if self.interpreter and not self.install_in_progress:
+                INSTALLATION_COMPLETED.fire()
+            else:
+                INSTALLATION_FAILED.fire()
+
+            self.destroy()
+
         if self.install_in_progress:
             widgets = []
 
@@ -246,13 +264,16 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
 
             self.installer_files_box.stop_all()
             if self.interpreter:
-                self.interpreter.revert(remove_game_dir=remove_checkbox.get_active())
+                self.interpreter.revert(
+                    remove_game_dir=remove_checkbox.get_active(),
+                    completion_function=on_cancelled,
+                    error_function=self.on_signal_error,
+                )
+            else:
+                on_cancelled()
         else:
             self.installer_files_box.stop_all()
-
-        if self.interpreter:
-            self.interpreter.cleanup()  # still remove temporary downloads in any case
-        self.destroy()
+            on_cancelled()
 
     def on_source_clicked(self, _button):
         InstallerSourceDialog(self.interpreter.installer.script_pretty, self.interpreter.installer.game_name, self)
@@ -270,10 +291,10 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
             display_error(error, parent=self)
             self.stack.navigation_reset()
 
-    def set_status(self, text):
+    def set_status(self, markup):
         """Display a short status text."""
-        self.status_label.set_text(text)
-        self.status_label.set_visible(bool(text))
+        self.status_label.set_markup(markup)
+        self.status_label.set_visible(bool(markup))
 
     def get_status(self):
         return self.status_label.get_text() if self.status_label.get_visible() else ""
@@ -288,6 +309,34 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
         self.stack.add_named_factory("error", self.create_error_page)
         self.stack.add_named_factory("nothing", lambda *x: Gtk.Box())
 
+    def load_first_page(self) -> None:
+        # If we're downloading updates in the background, we'll
+        # put up a spinner page to wait until that's done. Installations can
+        # fail if Lutris components are missing, and users sometimes try to install
+        # a game just after their first Lutris startup. This should help.
+        window = get_main_window()
+        if window and not window.download_queue.is_empty:
+            download_queue = window.download_queue
+
+            def on_start_installation(*args):
+                self.load_choose_installer_page()
+                download_queue.disconnect(dc_handler)
+
+            def on_download_complete(*args):
+                if download_queue.is_empty:
+                    on_start_installation()
+
+            dc_handler = download_queue.connect("download-completed", on_download_complete)
+            self.load_spinner_page(
+                _(
+                    "Waiting for Lutris component installation\n"
+                    "<small>Installations can fail if Lutris components are not installed first.</small>"
+                )
+            )
+            self.display_continue_button(on_start_installation)
+        else:
+            self.load_choose_installer_page()
+
     # Interpreter UI Delegate
     #
     # These methods are called from the ScriptInterpreter, and defer work until idle time
@@ -297,17 +346,17 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
         GLib.idle_add(self.error_reporter, error)
 
     def report_status(self, status):
-        GLib.idle_add(self.set_status, status)
+        GLib.idle_add(self.set_status, gtk_safe(status))
 
     def attach_log(self, command):
         # Hook the log buffer right now, lest we miss updates.
         command.set_log_buffer(self.log_buffer)
         GLib.idle_add(self.load_log_page)
 
-    def begin_disc_prompt(self, message, requires, installer, callback):
+    def begin_disc_prompt(self, message_markup, requires, installer, callback):
         GLib.idle_add(
             self.load_ask_for_disc_page,
-            message,
+            message_markup,
             requires,
             installer,
             callback,
@@ -317,7 +366,7 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
         GLib.idle_add(self.load_input_menu_page, alias, options, preselect, callback)
 
     def report_finished(self, game_id, status):
-        GLib.idle_add(self.load_finish_install_page, game_id, status)
+        GLib.idle_add(self.load_finish_install_page, game_id, gtk_safe(status))
 
     # Choose Installer Page
     #
@@ -537,8 +586,9 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
 
         self.set_status(
             _(
-                "This game has extra content. \nSelect which one you want and "
-                "they will be available in the 'extras' folder where the game is installed."
+                "This game has extra content\n"
+                "<small>Select which one you want and "
+                "they will be available in the 'extras' folder where the game is installed.</small>"
             )
         )
         self.stack.present_page("extras")
@@ -577,8 +627,8 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
         """Resume install when user has selected extras to download"""
         selected_extras = []
 
-        def save_extra(store, path, iter_):
-            selected, _inconsistent, extra, _label = store[iter_]
+        def save_extra(model, path, iter_):
+            selected, _inconsistent, extra, _label = model[iter_]
             if selected and extra:
                 selected_extras.append(extra)
 
@@ -601,7 +651,7 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
             patch_version = self.interpreter.installer.version
         else:
             patch_version = None
-
+        self.load_spinner_page(_("Preparing game files..."), cancellable=False)
         AsyncCall(
             self.interpreter.installer.prepare_game_files, self.on_files_prepared, self.selected_extras, patch_version
         )
@@ -681,7 +731,7 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
     # Provides a generic progress spinner and displays a status. The back button
     # is disabled for this page.
 
-    def load_spinner_page(self, status, cancellable=True, extra_buttons=None):
+    def load_spinner_page(self, status: str, cancellable: bool = True, extra_buttons: List[Gtk.Button] = None) -> None:
         def present_spinner_page():
             """Show a spinner in the middle of the view"""
 
@@ -694,7 +744,7 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
             if cancellable:
                 self.display_cancel_button(extra_buttons=extra_buttons)
             else:
-                self.display_buttons(extra_buttons or [])
+                self.display_buttons(extra_buttons or [], False)
 
             self.stack.set_back_allowed(False)
             return on_exit_page
@@ -725,7 +775,7 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
             self.error_reporter = saved_reporter
 
         def on_error(error):
-            self.set_status(str(error))
+            self.set_status(gtk_safe(error))
 
         saved_reporter = self.error_reporter
         self.error_reporter = on_error
@@ -787,7 +837,7 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
     # This page asks the user for a disc; it also has a callback used when
     # the user selects a disc. Again, this is summoned by the installer script.
 
-    def load_ask_for_disc_page(self, message, requires, installer, callback):
+    def load_ask_for_disc_page(self, message_markup, requires, installer, callback):
         def present_ask_for_disc_page():
             """Ask the user to do insert a CD-ROM."""
 
@@ -801,7 +851,7 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
                     self.load_error_page(err)
 
             vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
-            label = MarkupLabel(message)
+            label = MarkupLabel(message_markup)
             vbox.pack_start(label, False, False, 0)
 
             buttons_box = Gtk.Box()
@@ -857,16 +907,21 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
         self.cancel_button.grab_focus()
 
     def present_error_page(self, error: BaseException) -> None:
-        self.set_status(str(error))
+        self.set_status(gtk_safe(str(error)))
 
-        formatted = traceback.format_exception(type(error), error, error.__traceback__)
-        formatted = "\n".join(formatted).strip()
+        is_expected = hasattr(error, "is_expected") and error.is_expected
 
-        log = get_log_contents()
-        if log:
-            formatted = f"{formatted}\n\nLutris log:\n{log}".strip()
+        if is_expected:
+            formatted = traceback.format_exception(type(error), error, error.__traceback__)
+            formatted = "\n".join(formatted).strip()
 
-        self.error_details_buffer.set_text(formatted)
+            log = get_log_contents()
+            if log:
+                formatted = f"{formatted}\n\nLutris log:\n{log}".strip()
+
+            self.error_details_buffer.set_text(formatted)
+
+        self.error_details_box.set_visible(not is_expected)
 
         self.stack.present_page("error")
         self.display_cancel_button()
@@ -881,7 +936,8 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
             clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
             clipboard.set_text(text, -1)
 
-        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
+        error_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
+
         label = Gtk.Label(xalign=0.0, wrap=True)
         label.set_markup(
             _(
@@ -891,7 +947,8 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
                 "<a href='https://discordapp.com/invite/Pnt5CuY'>Discord</a>."
             )
         )
-        box.pack_start(label, False, False, 0)
+        self.error_details_box.pack_start(label, False, False, 0)
+
         frame = Gtk.Frame(shadow_type=Gtk.ShadowType.ETCHED_IN)
 
         details_textview = Gtk.TextView(editable=False, buffer=self.error_details_buffer)
@@ -899,13 +956,14 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
         scrolledwindow = Gtk.ScrolledWindow()
         scrolledwindow.add(details_textview)
         frame.add(scrolledwindow)
-        box.pack_start(frame, True, True, 0)
+        self.error_details_box.pack_start(frame, True, True, 0)
+        error_box.pack_start(self.error_details_box, True, True, 0)
 
-        copy_button = Gtk.Button(_("Copy Details to Clipboard"), halign=Gtk.Align.START)
-        box.pack_end(copy_button, False, True, 0)
+        copy_button = Gtk.Button(label=_("Copy Details to Clipboard"), halign=Gtk.Align.START)
+        error_box.pack_end(copy_button, False, True, 0)
         copy_button.connect("clicked", on_copy_clicked)
 
-        return box
+        return error_box
 
     # Finished Page
     #
@@ -1013,9 +1071,9 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
         )
 
     def display_cancel_button(self, extra_buttons=None):
-        self.display_buttons(extra_buttons or [])
+        self.display_buttons(extra_buttons or [], True)
 
-    def display_buttons(self, buttons):
+    def display_buttons(self, buttons, cancel_sensitive=True):
         """Shows exactly the buttons given, and hides the others. Updates the close button
         according to whether the install has started."""
 
@@ -1030,6 +1088,8 @@ class InstallerWindow(ModelessDialog, DialogInstallUIDelegate, ScriptInterpreter
             self.cancel_button.set_tooltip_text("")
             style_context.remove_class("destructive-action")
 
+        self.cancel_button.set_sensitive(cancel_sensitive)
+
         all_buttons = [self.cache_button, self.source_button, self.continue_button]
 
         for b in all_buttons:
