@@ -4,8 +4,10 @@
 # pylint: disable=no-member
 import os
 from collections import namedtuple
+from datetime import datetime
 from gettext import gettext as _
-from typing import Iterable, List
+from gettext import ngettext
+from typing import Dict, Iterable, List, Set, cast
 from urllib.parse import unquote, urlparse
 
 from gi.repository import Gdk, Gio, GLib, Gtk
@@ -15,8 +17,8 @@ from lutris.api import (
     LUTRIS_ACCOUNT_CONNECTED,
     LUTRIS_ACCOUNT_DISCONNECTED,
     get_runtime_versions,
-    read_user_info,
 )
+from lutris.database import categories
 from lutris.database import categories as categories_db
 from lutris.database import games as games_db
 from lutris.database import saved_searches as saved_searches_db
@@ -33,12 +35,23 @@ from lutris.gui.dialogs import ClientLoginDialog, ErrorDialog, QuestionDialog, g
 from lutris.gui.dialogs.delegates import DialogInstallUIDelegate, DialogLaunchUIDelegate
 from lutris.gui.dialogs.game_import import ImportGameDialog
 from lutris.gui.download_queue import DownloadQueue
+from lutris.gui.views import (
+    COL_INSTALLED_AT,
+    COL_INSTALLED_AT_TEXT,
+    COL_LASTPLAYED,
+    COL_LASTPLAYED_TEXT,
+    COL_NAME,
+    COL_PLAYTIME,
+    COL_PLAYTIME_TEXT,
+    COL_SORTNAME,
+    COL_YEAR,
+)
 from lutris.gui.views.grid import GameGridView
 from lutris.gui.views.list import GameListView
 from lutris.gui.views.store import GameStore
 from lutris.gui.widgets.game_bar import GameBar
 from lutris.gui.widgets.gi_composites import GtkTemplate
-from lutris.gui.widgets.sidebar import LutrisSidebar
+from lutris.gui.widgets.sidebar import LutrisSidebar, SidebarRow
 from lutris.gui.widgets.utils import has_stock_icon, load_icon_theme, open_uri
 from lutris.runtime import ComponentUpdater, RuntimeUpdater
 from lutris.search import GameSearch
@@ -59,7 +72,7 @@ from lutris.util.wine.wine import clear_wine_version_cache
 
 
 @GtkTemplate(ui=os.path.join(datapath.get(), "ui", "lutris-window.ui"))
-class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallUIDelegate):  # pylint: disable=too-many-public-methods
+class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallUIDelegate):  # type:ignore[misc]
     """Handler class for main window signals."""
 
     default_view_type = "grid"
@@ -81,12 +94,11 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
     game_view_spinner: Gtk.Spinner = GtkTemplate.Child()
     login_notification_revealer: Gtk.Revealer = GtkTemplate.Child()
     lutris_log_in_label: Gtk.Label = GtkTemplate.Child()
-    turn_on_library_sync_label: Gtk.Label = GtkTemplate.Child()
     version_notification_revealer: Gtk.Revealer = GtkTemplate.Child()
     version_notification_label: Gtk.Revealer = GtkTemplate.Child()
     show_hidden_games_button: Gtk.ModelButton = GtkTemplate.Child()
 
-    def __init__(self, application, **kwargs) -> None:
+    def __init__(self, application=None, **kwargs) -> None:
         width = int(settings.read_setting("width") or self.default_width)
         height = int(settings.read_setting("height") or self.default_height)
         super().__init__(
@@ -100,6 +112,7 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
         )
         update_desktop_icons()
         load_icon_theme()
+        self.set_wmclass("net.lutris.Lutris", "net.lutris.Lutris")
         self.application = application
         self.window_x, self.window_y = self.get_position()
         self.restore_window_position()
@@ -123,12 +136,13 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
             "running": self.get_running_games,
         }
 
+        for smart_category in categories._SMART_CATEGORIES:
+            if smart_category.get_name() not in self.dynamic_categories_game_factories:
+                self.dynamic_categories_game_factories[smart_category.get_name()] = smart_category.get_games
+
         self.accelerators = Gtk.AccelGroup()
         self.add_accel_group(self.accelerators)
 
-        key, mod = Gtk.accelerator_parse("F5")
-        self.accelerators.connect(key, mod, Gtk.AccelFlags.VISIBLE, self.on_refresh)
-
         self.connect("delete-event", self.on_window_delete)
         self.connect("configure-event", self.on_window_configure)
         self.connect("realize", self.on_load)
@@ -142,7 +156,7 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
         # Since system-search-symbolic is already *right there* we'll try to pick some
         # other icon for the button that shows the search popover.
         fallback_filter_icons_names = ["filter-symbolic", "edit-find-replace-symbolic", "system-search-symbolic"]
-        filter_button_image = self.search_filters_button.get_child()
+        filter_button_image: Gtk.Image = self.search_filters_button.get_child()
         for n in fallback_filter_icons_names:
             if has_stock_icon(n):
                 filter_button_image.set_from_icon_name(n, Gtk.IconSize.BUTTON)
@@ -298,9 +312,9 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
         def on_library_synced(_result, error):
             """Sync media after the library is loaded"""
             if not error:
-                sync_media()
+                AsyncCall(sync_media, None)
 
-        if settings.read_bool_setting("library_sync_enabled"):
+        if settings.read_bool_setting("library_sync_enabled", True):
             AsyncCall(LibrarySyncer().sync_local_library, on_library_synced if force else None, force=force)
 
     def update_action_state(self):
@@ -423,6 +437,21 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
         """True if the view sorting options will be effective; dynamic categories ignore them."""
         return self.filters.get("dynamic_category") not in self.dynamic_categories_game_factories
 
+    def get_sort_sensitive_columns(self) -> Set[int]:
+        if self.is_view_sort_sensitive:
+            if self.view_sorting == "name":
+                return set([COL_NAME, COL_SORTNAME])
+            elif self.view_sorting == "year":
+                return set([COL_YEAR])
+            elif self.view_sorting == "lastplayed":
+                return set([COL_LASTPLAYED, COL_LASTPLAYED_TEXT])
+            elif self.view_sorting == "installed_at":
+                return set([COL_INSTALLED_AT, COL_INSTALLED_AT_TEXT])
+            elif self.view_sorting == "playtime":
+                return set([COL_PLAYTIME, COL_PLAYTIME_TEXT])
+
+        return set()
+
     def apply_view_sort(self, items, resolver=lambda i: i):
         """This sorts a list of items according to the view settings of this window;
         the items can be anything, but you can provide a lambda that provides a
@@ -442,36 +471,71 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
             "playtime": 0.0,
         }
 
+        def get_sort_default(item):
+            """Returns the default value to use when the value is missing; we may be able
+            to extract this from the item.."""
+            if self.view_sorting == "year" and self.service:
+                service_year = self.service.get_game_release_date(item)
+                service_year = convert_value(service_year)
+                if service_year:
+                    return service_year
+
+            # Users may have obsolete view_sorting settings, so
+            # we must tolerate them. We treat them all as blank.
+            return sort_defaults.get(self.view_sorting, "")
+
+        def convert_value(value):
+            """Converts 'value' to the type required for the sort that is in use. Returns None if this
+            can't be managed."""
+            try:
+                if not value:
+                    return None
+                if self.view_sorting == "name":
+                    return str(value)
+                if self.view_sorting == "year":
+                    # Years can take many forms! We'll try to convert as best we can.
+                    if isinstance(value, datetime):
+                        return int(value.year)
+                    else:
+                        try:
+                            return int(value)
+                        except ValueError:
+                            as_date = datetime.strptime(str(value), "%Y-%m-%d")
+                            return int(as_date.year)
+                else:
+                    return float(value)
+            except ValueError:
+                return None  # unable to parse value?
+
+        def extend_value(value):
+            """Expands the value to sort by to a more complex form, for smarter sorting."""
+            if self.view_sorting == "name":
+                return get_natural_sort_key(value)
+            if self.view_sorting == "year":
+                contains_year = bool(value)
+                if self.view_reverse_order:
+                    contains_year = not contains_year
+                return contains_year, value
+            return value
+
         def get_sort_value(item):
             db_game = resolver(item)
             if not db_game:
                 installation_flag = False
-                value = sort_defaults.get(self.view_sorting, "")
+                value = None
             else:
                 installation_flag = bool(db_game.get("installed"))
 
                 # When sorting by name, check for a valid sortname first, then fall back
                 # on name if valid sortname is not available.
-                sortname = db_game.get("sortname")
-                if self.view_sorting == "name" and sortname:
-                    value = sortname
+                if self.view_sorting == "name":
+                    value = db_game.get("sortname") or db_game.get("name")
                 else:
                     value = db_game.get(self.view_sorting)
 
-                if self.view_sorting == "name":
-                    value = get_natural_sort_key(value)
-            # Users may have obsolete view_sorting settings, so
-            # we must tolerate them. We treat them all as blank.
-            value = value or sort_defaults.get(self.view_sorting, "")
-            if self.view_sorting == "year":
-                if self.service:
-                    service_value = self.service.get_game_release_date(item)
-                    if service_value:
-                        value = service_value
-                contains_year = bool(value)
-                if self.view_reverse_order:
-                    contains_year = not contains_year
-                value = [contains_year, value]
+            value = convert_value(value) or get_sort_default(item)
+            value = extend_value(value)
+
             if self.view_sorting_installed_first:
                 # We want installed games to always be first, even in
                 # a descending sort.
@@ -479,7 +543,7 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
                     installation_flag = not installation_flag
                 if self.view_sorting == "name":
                     installation_flag = not installation_flag
-                return [installation_flag, value]
+                return installation_flag, value
             return value
 
         reverse = self.view_reverse_order if self.view_sorting == "name" else not self.view_reverse_order
@@ -593,7 +657,7 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
         service_id = self.filters.get("service")
         if service_id in services.SERVICES:
             if self.service.online and not self.service.is_authenticated():
-                self.show_label(_("Connect your %s account to access your games") % self.service.name)
+                self.show_empty_label()
                 return []
             return self.get_service_games(service_id)
         if self.filters.get("dynamic_category") in self.dynamic_categories_game_factories:
@@ -621,11 +685,22 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
         category_game_ids = categories_db.get_game_ids_for_categories(included, excluded)
 
         filters = self.get_sql_filters()
-        games = games_db.get_games(filters=filters)
+        excludes = {}
+
+        if self.filters.get("category") == "all" and "service" not in self.filters:
+            excluded_services = set(
+                s.casefold()
+                for s in services.SERVICES.keys()
+                if not settings.read_bool_setting(s + "_in_games_view", default=True, section="services")
+            )
+            if excluded_services:
+                excludes["service"] = excluded_services
+
+        games = games_db.get_games(filters=filters, excludes=excludes)
         games = self.filter_games([game for game in games if game["id"] in category_game_ids], searches=searches)
         return self.apply_view_sort(games)
 
-    def get_sql_filters(self):
+    def get_sql_filters(self) -> Dict[str, str]:
         """Return the current filters for the view"""
         sql_filters = {}
         if self.filters.get("runner"):
@@ -670,6 +745,10 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
 
     def show_empty_label(self):
         """Display a label when the view is empty"""
+        if self.service and self.service.online and not self.service.is_authenticated():
+            self.show_label(_("Connect your %s account to access your games") % self.service.name)
+            return
+
         filter_text = self.filters.get("text")
         has_uninstalled_games = games_db.get_game_count("installed", "0")
         if filter_text:
@@ -741,25 +820,26 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
 
             games, game_store = result
 
-            if games:
-                if len(games) > 1:
-                    self.search_entry.set_placeholder_text(_("Search %s games") % len(games))
-                else:
-                    self.search_entry.set_placeholder_text(_("Search 1 game"))
-            else:
-                self.search_entry.set_placeholder_text(_("Search games"))
+            placeholder_text = self._get_search_placeholder_text(games)
+            self.search_entry.set_placeholder_text(placeholder_text)
 
             for view in self.views.values():
                 view.service = self.service
 
             GLib.idle_add(self.update_revealer)
-            self.game_store = game_store
 
-            view_type = self.current_view_type
+            if self.game_store != game_store:
+                self.game_store = game_store
 
-            if view_type in self.views:
-                self.current_view = self.views[view_type]
-                self.current_view.set_game_store(self.game_store)
+                view_type = self.current_view_type
+
+                if view_type in self.views:
+                    view = self.views[view_type]
+                    self.current_view = view
+                    selected_ids = [view.get_game_id_for_path(p) for p in view.get_selected()]
+                    view.set_game_store(self.game_store)
+                    new_selection = [view.get_path_for_game_id(game_id) for game_id in selected_ids]
+                    view.set_selected(filter(None, new_selection), scroll_into_view=True)
 
             if games:
                 self.hide_overlay()
@@ -770,6 +850,14 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
 
         AsyncCall(self.get_games_from_filters, on_games_ready)
 
+    @staticmethod
+    def _get_search_placeholder_text(games) -> str:
+        if not games:
+            return _("Search games")
+
+        games_count = len(games)
+        return ngettext("Search %d game", "Search %d games", games_count) % games_count
+
     def _bind_zoom_adjustment(self):
         """Bind the zoom slider to the supported banner sizes"""
         service = self.service if self.service else LutrisService
@@ -978,26 +1066,17 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
     def update_notification(self):
         show_notification = self.is_showing_splash()
         if show_notification:
-            if not read_user_info():
-                self.lutris_log_in_label.show()
-                self.turn_on_library_sync_label.hide()
-            elif not settings.read_bool_setting("library_sync_enabled"):
-                self.lutris_log_in_label.hide()
-                self.turn_on_library_sync_label.show()
-            else:
-                show_notification = False
-
+            self.lutris_log_in_label.show()
         self.login_notification_revealer.set_reveal_child(show_notification)
 
     @GtkTemplate.Callback
     def on_lutris_log_in_label_activate_link(self, _label, _url):
-        ClientLoginDialog(parent=self)
+        def on_connect_success(widget, _username):
+            self.sync_library(force=True)
 
-    @GtkTemplate.Callback
-    def on_turn_on_library_sync_label_activate_link(self, _label, _url):
-        settings.write_setting("library_sync_enabled", True)
-        self.sync_library(force=True)
-        self.update_notification()
+        self.login_notification_revealer.set_reveal_child(False)
+        login_dialog = ClientLoginDialog(parent=self)
+        login_dialog.connect("connected", on_connect_success)
 
     def on_version_notification_close_button_clicked(self, _button):
         dialog = QuestionDialog(
@@ -1197,10 +1276,6 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
         settings.write_setting("side_panel_visible", bool(side_panel_visible))
         self.sidebar_revealer.set_reveal_child(side_panel_visible)
 
-    def on_refresh(self, _accel_group, _window, _keyval, _modifier):
-        """Handle F5 key, which updates the view explicitly."""
-        self.refresh_view()
-
     def on_sidebar_changed(self, widget):
         """Handler called when the selected element of the sidebar changes"""
         for filter_type in ("category", "dynamic_category", "saved_search", "service", "runner", "platform"):
@@ -1253,9 +1328,11 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
         settings.write_setting("hide_badges_on_icons", not state)
         self.on_settings_changed(None, not state, "hide_badges_on_icons")
 
-    def on_settings_changed(self, setting_key, new_value):
-        if setting_key == "hide_text_under_icons":
+    def on_settings_changed(self, setting_key, new_value, section):
+        if section == "lutris" and setting_key == "hide_text_under_icons":
             self.rebuild_view("grid")
+        elif section == "services" and setting_key.endswith("_in_games_view"):
+            self.update_store()
         else:
             self.update_view_settings()
         self.update_notification()
@@ -1302,8 +1379,9 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
                 return True
 
         if db_game:
-            updated = self.game_store.update(db_game)
-            if not updated:
+            updated_columns = self.game_store.update(db_game)
+            sensitive_columns = self.get_sort_sensitive_columns()
+            if updated_columns is None or not sensitive_columns.isdisjoint(updated_columns):
                 self.update_store()
 
         return True
@@ -1313,7 +1391,7 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
         selected_row = self.sidebar.get_selected_row()
         # Only update the running page- we lose the selected row when we do this,
         # but on the running page this is okay.
-        if selected_row is not None and selected_row.id == "running":
+        if isinstance(selected_row, SidebarRow) and selected_row.id == "running":
             self.game_store.remove_game(game.id)
 
     def on_game_installed(self, game):
@@ -1344,7 +1422,7 @@ class LutrisWindow(Gtk.ApplicationWindow, DialogLaunchUIDelegate, DialogInstallU
 
     @property
     def download_queue(self) -> DownloadQueue:
-        queue = self.download_revealer.get_child()
+        queue = cast(DownloadQueue, self.download_revealer.get_child())
         if not queue:
             queue = DownloadQueue(self.download_revealer)
             self.download_revealer.add(queue)
