@@ -2,7 +2,7 @@
 
 import array
 import os
-from typing import Optional
+from typing import TYPE_CHECKING, Iterable, List, Optional, Type, TypeVar, cast
 
 import cairo
 from gi.repository import Gdk, GdkPixbuf, Gio, GLib, Gtk
@@ -13,6 +13,10 @@ from lutris.gui.widgets import NotificationSource
 from lutris.util import datapath, magic, system
 from lutris.util.log import logger
 
+if TYPE_CHECKING:
+    from lutris.gui.application import LutrisApplication
+    from lutris.gui.lutriswindow import LutrisWindow
+
 try:
     from PIL import Image
 except ImportError:
@@ -23,16 +27,60 @@ BANNER_SIZE = (184, 69)
 MEDIA_CACHE_INVALIDATED = NotificationSource()
 
 
-def get_main_window(widget):
-    """Return the application's main window from one of its widget"""
-    parent = widget.get_toplevel()
-    if not isinstance(parent, Gtk.Window):
-        # The sync dialog may have closed
-        parent = Gio.Application.get_default().props.active_window
-    for window in parent.application.get_windows():
-        if "LutrisWindow" in window.__class__.__name__:
-            return window
-    return
+def get_application() -> Optional["LutrisApplication"]:
+    return cast("LutrisApplication", Gio.Application.get_default())
+
+
+def get_required_application() -> "LutrisApplication":
+    application = cast("LutrisApplication", Gio.Application.get_default())
+    if not application:
+        raise RuntimeError("The LutrisApplication does not exist.")
+    return application
+
+
+def get_main_window() -> Optional["LutrisWindow"]:
+    """Return the application's main window, or None if it doesn't exist
+    (though it almost alway does exist)"""
+    application = get_application()
+    return application.window if application else None
+
+
+def get_required_main_window() -> "LutrisWindow":
+    """Return the application's main window or raises an exception if
+    it doesn't exist."""
+    application = get_required_application()
+    window = application.window
+    if not window:
+        raise RuntimeError("The main window does not exist.")
+    return window
+
+
+def get_widget_window(widget: Optional[Gtk.Widget]) -> Optional[Gtk.Window]:
+    """Returns the window that contains a widget, if any. This wll return None
+    for a widget that is not in a window, rather than returning the widget itself
+    like get_toplevel()."""
+    if widget:
+        return cast(Optional[Gtk.Window], widget.get_ancestor(Gtk.Window))
+    else:
+        return None
+
+
+TChildWidget = TypeVar("TChildWidget", bound=Gtk.Widget)
+
+
+def get_widget_children(
+    widget: Optional[Gtk.Widget], child_type: Optional[Type[TChildWidget]] = None
+) -> List[TChildWidget]:
+    """Returns the children of any widget; non-containers have no children
+    and returns an empty list. This can filter out a specific type of child widget if child_type
+    is not None, but otherwise it returns all children."""
+    if isinstance(widget, Gtk.Container):
+        if child_type:
+            return [cast(TChildWidget, w) for w in widget.get_children() if isinstance(w, child_type)]
+        else:
+            return list(cast(Iterable[TChildWidget], widget.get_children()))
+    else:
+        return []
 
 
 def open_uri(uri):
@@ -98,8 +146,8 @@ def get_scaled_surface_by_path(path, size, device_scale, preserve_aspect_ratio=T
         scale_x = min(scale_x, scale_y)
         scale_y = scale_x
 
-    pixel_width = int(round(pixbuf_width * scale_x))
-    pixel_height = int(round(pixbuf_height * scale_y))
+    pixel_width = round(pixbuf_width * scale_x)
+    pixel_height = round(pixbuf_height * scale_y)
 
     surface = cairo.ImageSurface(cairo.Format.ARGB32, pixel_width, pixel_height)  # pylint:disable=no-member
     cr = cairo.Context(surface)  # pylint:disable=no-member
